
// ReadGPS.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, November 2010

/* Use the gpsinput package from gpsylon (http://www.tegmento.org/gpsylon/)
   to connect to a GPS receiver pretending to be a serial port
   (the Canmore GT-730F USB GPS receiver, http://www.canmore.com.tw/)
   and report all messages and all LOCATION GPS events.

   The GPS processing assumes that the receiver is dealing with
   NMEA messages. The NMEA format is described at
      http://www.gpsinformation.org/dale/nmea.htm

   For a more complex example see GPSTool.java in the gpsylon download

   Usage:
     run ReadGPS COM3

*/

import java.io.*;
import java.util.Hashtable;
import java.beans.*;    // for detecting property change

import org.dinopolis.gpstool.gpsinput.*;
import org.dinopolis.gpstool.gpsinput.nmea.GPSNmeaDataProcessor;


public class ReadGPS
{
  private static final int BAUD_RATE = 4800;

  public static void main(String[] argc)
  {
    if (argc.length != 1) {
      System.out.println("Usage: run ReadGPS <com>");
      System.exit(0);
    }

	try {
      // create processor for NMEA data
      final GPSDataProcessor gpsDataProc = new GPSNmeaDataProcessor();

      // create GPS device for serial port
      Hashtable<String, Object> env = new Hashtable<String, Object>();
	  env.put(GPSSerialDevice.PORT_NAME_KEY, argc[0]);
	  env.put(GPSSerialDevice.PORT_SPEED_KEY, new Integer(BAUD_RATE));
	  GPSDevice gpsDev = new GPSSerialDevice();
	  gpsDev.init(env);

      // connect processor to device and open it
	  gpsDataProc.setGPSDevice(gpsDev);
	  gpsDataProc.open();
	  System.out.println ("Opened GPS device");

      // print out device info
      System.out.println("GPS Info:");
      String[] info = gpsDataProc.getGPSInfo();
      for(int i=0; i < info.length; i++)
        System.out.println("  " + info[i]);

      // create a raw data listener, to print everything
      gpsDataProc.addGPSRawDataListener( new GPSRawDataListener() {
        public void gpsRawDataReceived(char[] data, int offset, int length)
        {  
          String msg = new String(data,offset,length).trim();
          System.out.println("Raw data (" + msg.length() + "): [" + msg + "]");
        }
      });

      // create a listener for GPS location events
	  PropertyChangeListener listener = new PropertyChangeListener () {
		public void propertyChange(PropertyChangeEvent event) 
        {
		  Object value = event.getNewValue();
	      String name = event.getPropertyName();
          System.out.println("Property Name: " + name);
		  if (name.equals(GPSDataProcessor.LOCATION)) {
			GPSPosition gpspos = (GPSPosition)value;
			System.out.println ("Lat | long: "+ gpspos.getLatitude() +" | " + 
                                                gpspos.getLongitude() ) ;
		  }
		}
      };
	  gpsDataProc.addGPSDataChangeListener(gpsDataProc.LOCATION, listener);
      /* other GPS events that can be monitored include
           GPSDataProcessor.SPEED, GPSDataProcessor.HEADING,
           GPSDataProcessor.NUMBER_SATELLITES,
          GPSDataProcessor.SATELLITE_INFO,
      */
      // gpsDataProc.startSendPositionPeriodically(1000L);

      // close device and processor:
      // gpsDataProc.close();
    }
    catch(GPSException e)
    {  e.printStackTrace(); }
    catch(Exception e)
    {  System.err.println(e); }
  }  // end of main()

}  // end of ReadGPS class

 