
// SerialBufReader.java
// Andrew Davison, November 2010, ad@fivedots.coe.psu.ac.th

/* Open a serial port using the RXTX implementation of the
   Java Communications API (http://www.rxtx.org/),
   and receive messages. 

   The port is actually a GPS receiver
   (the Canmore GT-730F USB GPS receiver, http://www.canmore.com.tw/)
   which is receiving NMEA formatted data. The NMEA format is 
   described at
      http://www.gpsinformation.org/dale/nmea.htm

   The messages are separated by newlines, and are written in ASCII,
   and so the serial port can be treated as a BufferedReader.

   Only NMEA location messages are processed (those starting with
   $GPGGA), and the latutude and longitude details are printed out
   by showPosition().
   
   Usage:
     run SerialBufReader COM3

   The user must supply the serial port ID (COM3), but the code assumes
   a baud rate of 4800.

 */

import java.util.*;
import java.io.*;
import gnu.io.*;   // using RXTX 2.1.7


public class SerialBufReader implements SerialPortEventListener
{
  private static final int BAUD_RATE = 4800;

  private BufferedReader br;  // for reading the NMEA messages


  public SerialBufReader(String comStr)
  { 
    // covert the serial port name into an ID
    CommPortIdentifier portId = null;
    try {
      portId = CommPortIdentifier.getPortIdentifier(comStr);
    }
    catch(NoSuchPortException e) {
      System.out.println("Could not find port: " + comStr);
      System.exit(0);
    }

    // check that it's a serial port
    if (portId.getPortType() != CommPortIdentifier.PORT_SERIAL) {
      System.out.println(comStr + " not a serial port");
      System.exit(0);
    }

    // open the port
    SerialPort serialPort = null;
    try {
      serialPort = (SerialPort) portId.open("SerialBufReader", 2000);
    }
    catch (PortInUseException e) {
      System.out.println(comStr + " already in use");
      System.exit(0);
    }

    // use this class as the serial port listener; serialEvent() will be called
    try {
      serialPort.addEventListener(this);
    }
    catch (TooManyListenersException e) {
      System.out.println(comStr + " has listener already");
      System.exit(0);
    }

    serialPort.notifyOnDataAvailable(true);  // listen for data availability

    try {
	  serialPort.enableReceiveTimeout(1000);    // wait for timeout
	  serialPort.enableReceiveThreshold(0);
      serialPort.setSerialPortParams(BAUD_RATE, SerialPort.DATABITS_8,
                SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);
    }
    catch (UnsupportedCommOperationException e) {
      System.out.println(comStr + ": error while setting parameters");
      System.exit(0);
    }

    // layer a BufferedReader on top of the port
    try {
      br = new BufferedReader( new InputStreamReader(
                                  serialPort.getInputStream() ));
    }
    catch (IOException e) {
      System.out.println("Could not create buffered input reader for: " + comStr);
      System.exit(0);
    }

    System.out.println("Openned port " + comStr + " for input");
  }  // end of SerialBufReader()




  public void serialEvent(SerialPortEvent event)
  /* this method should only be called when data becomes 
     available on the port. Print position details.
  */
  {
    if (event.getEventType() == SerialPortEvent.DATA_AVAILABLE) {
      try {
        String msg = br.readLine();  // get a single NMEA message
        if (msg == null) {
          System.out.println("input terminated");
          br.close();
          System.exit(0);
        }
        else {
          System.out.println("Msg len: " + msg.length() + "; [" + msg + "]");
          showPosition(msg);
        }
      }
      catch(IOException e) {
        System.out.println(e);
        System.exit(0);
      }
    }
    else
      System.out.println("Port event received: " + event);
  }  // end of serialEvent()


  private void showPosition(String msg)
  /* Only NMEA location messages are processed (those starting with
     $GPGGA), and the latutude and longitude details are printed out.
     For details see:
       http://www.gpsinformation.org/dale/nmea.htm#GGA
  */
  {
    String[] nmeaArgs = msg.split(",");  
        // the arguments of all NMEA messages are delimited by ","s
    if (nmeaArgs.length >= 6) {
      if (nmeaArgs[0].equals("$GPGGA")) // position fix NMEA sentence
        System.out.println("Lat: " + nmeaArgs[2] + " " + nmeaArgs[3] + 
                         "; Long: " + nmeaArgs[4] + " " + nmeaArgs[5]);
    }
  }  // end of showPosition()


  // -------------------------------------------------

  public static void main(String[] argc)
  {
    if (argc.length != 1)
      System.out.println("Usage: run SerialBufReader <com name>");
    else
      new SerialBufReader(argc[0]);
  }  // end of main()


} // end of SerialBufReader class
