
// MetaDataUtils.java
// Andrew Davison, November 2010, ad@fivedots.coe.psu.ac.th

/* Some utility methods to simplify the reading and writing of
   GPS metadata stored in EXIF format in a JPEG or TIFF file by 
   using Sanselan (http://commons.apache.org/sanselan/)

   This code is partly based on the example code, 
   MetadataExample and WriteExifMetadataExample in 
   org.apache.sanselan.sampleUsage in the Sanselan source distribution.

   Also ueful was the blog post:
     "Reading and writing JPEG metadata (EXIF) from Java with Sanselan"
     http://www.screaming-penguin.com/node/7485
 */

import java.io.*;

import org.apache.sanselan.*;
import org.apache.sanselan.common.*;
import org.apache.sanselan.formats.jpeg.JpegImageMetadata;
import org.apache.sanselan.formats.tiff.*;
import org.apache.sanselan.formats.tiff.constants.*;
import org.apache.sanselan.formats.tiff.TiffImageMetadata.GPSInfo;
import org.apache.sanselan.formats.tiff.write.*;
import org.apache.sanselan.formats.jpeg.exifRewrite.ExifRewriter;


public class MetaDataUtils
{

  public static JpegImageMetadata getJpegMD(String fnm) 
  // access the file's JPG meta data
  {
    System.out.println("Looking for JPEG metadata in " + fnm);
    IImageMetadata md = null;
    try {
      md = Sanselan.getMetadata( new File(fnm));
    }
    catch(Exception e)
    {  System.out.println(e);  }

    if (md == null) {
      System.out.println("No metadata found");
      return null;
    }

    if (!(md instanceof JpegImageMetadata)) {
      System.out.println("Metadata is not for JPEG/TIFF");
      return null;
    }
    return (JpegImageMetadata) md;
  }  // end of getJpegMD()



  public static double[] getGPSCoord(String fnm)
  // return the (lat, long) GPS data from the file
  {
    TiffImageMetadata.GPSInfo gpsInfo = MetaDataUtils.getGPSInfo(fnm);
    if (gpsInfo == null)
      return null;

    try {
      double longitude = gpsInfo.getLongitudeAsDegreesEast();
      double latitude = gpsInfo.getLatitudeAsDegreesNorth();
      return new double[] { latitude, longitude };
    }
    catch(ImageReadException e) {
      System.out.println("Could not read latitude and longitude from " + fnm);
    }
    return null;
  }  // end of getGPSCoord()



  private static TiffImageMetadata.GPSInfo getGPSInfo(String fnm)
  // get the GPS data from the file
  {
    // access the file's meta data
    JpegImageMetadata jpegMD = MetaDataUtils.getJpegMD(fnm);
    if (jpegMD == null)
      return null;

    TiffImageMetadata exifMD = jpegMD.getExif();
    if (exifMD == null) {
      System.out.println("No EXIF metadata found");
      return null;
    }

    TiffImageMetadata.GPSInfo gpsInfo = null;
    try {
      gpsInfo = exifMD.getGPS();
    }
    catch(ImageReadException e)
    {  System.out.println("Could not read GPS Information");  }

    return gpsInfo;
  }  // end of getGPSInfo()



  public static TiffField getTag(String fnm, TagInfo tagInfo)
  {
    // access the file's meta data
    JpegImageMetadata jpegMD = MetaDataUtils.getJpegMD(fnm);
    if (jpegMD == null)
      return null;
    return jpegMD.findEXIFValue(tagInfo);
  }  // end of printTag()



  // ------------------- output utilities --------------------


  public static boolean putGPSInfo(String fnm, double lat, double lon, String outFnm)
  // modify the (lat,lon) GPS data in fnm, saving it in outFnm
  {
    File inFile = null;
    try {
      inFile = new File(fnm);
    }
    catch(Exception e)
    {  System.out.println(e);  
       return false;
    }

    // open output stream to a new file
    BufferedOutputStream bout = null;
    try {
      bout = new BufferedOutputStream(new FileOutputStream(outFnm));
    } 
    catch (Exception e) 
    { System.out.println(e);  
      return false;
    } 

    // create a TiffOutputSet for holding the new GPS data
    TiffOutputSet outSet = null;
    try {
      // access the input file's meta data (may be null)
      JpegImageMetadata jpegMD = MetaDataUtils.getJpegMD(fnm);
      if (jpegMD != null) {
        TiffImageMetadata exif = jpegMD.getExif();  // copy Exif metadata
        if (exif != null)
          outSet = exif.getOutputSet();
      }
      // if file does not contain metadata, create an empty TiffOutputSet
      if (outSet == null)
        outSet = new TiffOutputSet();

      outSet.setGPSInDegrees(lon, lat);
    }
    catch (Exception e) 
    { System.out.println(e);  
      return false;
    } 
   
    // write fnm contents and the modified metadata to the new file
    try {
      new ExifRewriter().updateExifMetadataLossless(inFile, bout, outSet);
      bout.flush();
      bout.close();
      return true;
    }
    catch (Exception e) 
    { System.out.println(e);} 

    return false;
  }  // end of putGPSInfo()


}  // end of MetaDataUtils class
