
// ShowGPS.java
// Andrew Davison, November 2010, ad@fivedots.coe.psu.ac.th

/* Use the gpsinput package from gpsylon (http://www.tegmento.org/gpsylon/)
   to connect to a GPS receiver pretending to be a serial port
   (the Canmore GT-730F USB GPS receiver, http://www.canmore.com.tw/)
   and report all LOCATION GPS events.

   The GPS processing assumes that the receiver is dealing with
   NMEA messages. The NMEA format is described at
      http://www.gpsinformation.org/dale/nmea.htm

   The current latitude and longitude are displayed
   on a map downloaded from OpenStreetMap 
   (http://www.openstreetmap.org/).

   This is managed by a JMapViewer Swing component, which
   is a subclass of JPanel. JMapViewer is explained at
   http://wiki.openstreetmap.org/wiki/JMapViewer.

   JMapViewer can be downloaded from
     http://svn.openstreetmap.org/applications/viewer/jmapviewer
   
   Usage:
      run ShowGPS COM3

   Use Google maps with lat, lon to check positions.

   This program combines ideas from ReadGPS.java and ShowMap.java

   There is some commented-out code for reducing the updates to the
   map display. The map is only updated when the marker has moved
   a certain distance (MIN_MOVE_DIST) from its current position.
*/

import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.util.Hashtable;
import java.beans.*;    // for property change

import org.openstreetmap.gui.jmapviewer.*;

import org.dinopolis.gpstool.gpsinput.*;
import org.dinopolis.gpstool.gpsinput.nmea.GPSNmeaDataProcessor;



public class ShowGPS extends JFrame 
                         implements PropertyChangeListener
{
  // hardwired serial port name and baud rate
  private static final String COM_PORT = "COM3";
  private static final int BAUD_RATE = 4800;

  // default marker position (the Robot building)
  private static final double LAT_DEFAULT = 7.006835; 
  private static final double LONG_DEFAULT = 100.5020;

  // private static final double MIN_MOVE_DIST = 2;   // in meters


  private JMapViewer map;   // the map display panel

  private MapMarkerDot currPos;    // current location on the map
  private double currLat, currLong;



  public ShowGPS(String comStr) 
  {
    super("Show GPS");

    Container c = getContentPane();
    c.setLayout( new BorderLayout() );

    map = new JMapViewer();    // map panel, default size is 400x400
    c.add(map, BorderLayout.CENTER);

    // start with a default location
    currLat = LAT_DEFAULT;
    currLong = LONG_DEFAULT;
    System.out.printf("Starting Position: (%.6f, %.6f)\n", currLat, currLong);

    currPos = new MapMarkerDot(currLat, currLong);
    map.addMapMarker(currPos);
    map.setDisplayToFitMapMarkers();  // center the map on the marker
    map.setZoom(17);  // so roads are visible
    map.setZoomContolsVisible(false);

    monitorGPS(comStr);    // start monitoring the GPS receiver

    setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
    pack();
    setVisible(true);
  }  // end of ShowGPS()


  private void monitorGPS(String comStr)
  /* start monitoring the GPS receiver for location events;
     call propertyChange() when one arrives */
  {
    try {
      // create processor for NMEA data
      final GPSDataProcessor gpsDataProc = new GPSNmeaDataProcessor();

      // create GPS device for serial port
      Hashtable<String, Object> env = new Hashtable<String, Object>();
      env.put(GPSSerialDevice.PORT_NAME_KEY, comStr);
      env.put(GPSSerialDevice.PORT_SPEED_KEY, new Integer(BAUD_RATE));
      GPSDevice gpsDev = new GPSSerialDevice();
      gpsDev.init(env);

      // connect processor to device and open it
      gpsDataProc.setGPSDevice(gpsDev);
      gpsDataProc.open();
      System.out.println ("Opened GPS device");
/*
      // create a raw data listener, to print everything (useful for debugging)
      gpsDataProc.addGPSRawDataListener( new GPSRawDataListener() {
        public void gpsRawDataReceived(char[] data, int offset, int length)
        {  
          String msg = new String(data,offset,length).trim();
          System.out.println("Raw data (" + msg.length() + "): [" + msg + "]");
        }
      });
*/
      // create listener for GPS location events
      gpsDataProc.addGPSDataChangeListener(gpsDataProc.LOCATION, this);
    }
    catch(Exception e)
    {  System.err.println(e); 
       System.exit(0);
    }
  }  // end of monitorGPS()


  public void propertyChange(PropertyChangeEvent event) 
  // called when the GPS receiver gets a location event
  {
    Object value = event.getNewValue();
    String name = event.getPropertyName();
    // System.out.println("Property Name: " + name);

    if (name.equals(GPSDataProcessor.LOCATION)) {
      GPSPosition gpspos = (GPSPosition)value;
      double newLat = gpspos.getLatitude();
      double newLong = gpspos.getLongitude();
      System.out.printf("Position: (%.6f, %.6f)\n", newLat, newLong);
      
      if ((newLat == 0) && (newLong == 0)) {   // ignore (0,0) position
        // System.out.println ("Ignoring (0,0) position") ;
        return;
      }
/*  
      // reduce updates to the map display by checking distance moved
      double distMoved = GeoMath.distanceApart(newLat, newLong, currLat, currLong);
      System.out.printf("Distance moved (in meters): %.2f\n", distMoved);
      if (distMoved < MIN_MOVE_DIST) { 
        System.out.println ("Ignoring small move") ;
        return;
      }
*/
      // update the map position
      currLat = newLat; currLong = newLong;
      map.removeMapMarker(currPos);
      currPos = new MapMarkerDot(newLat, newLong);
      map.addMapMarker(currPos);
      map.setDisplayToFitMapMarkers();
      map.repaint();
    }
  }  // end of propertyChange()


    // -------------------------------------------------


  public static void main(String[] argc) 
  {  
    if (argc.length == 0) {
      System.out.println("Using port: " + COM_PORT);
      new ShowGPS(COM_PORT);
    }
    else if (argc.length == 1)
      new ShowGPS(argc[0]);
    else
      System.out.println("Usage: run ShowGPS [<com>]");

  }  // end of main()

}  // end of ShowGPS class
