
// StitchTiles.java
// Andrew Davison, November 2010, ad@fivedots.coe.psu.ac.th

/* An example of downloading a series of map tiles from
   OpenStreetMap (http://www.openstreetmap.org/) 
   without rendering them. The tiles are centered around a 
   latitude and longitude supplied by the user.

   The tiles are downloaded concurrently. 

   The tiles are stored in the local subdirectory TILES_DIR 
   as PNG files, and loaded into a TileCache data structure in
   memory. The tile images are 'stitched together' to form
   a single large image, stored in IMG_FNM.

   The storage of the files locally means that if the same query is
   run again, then the code will use the local tiles rather than download
   them over the Web, with a clear increase in speed.

   This code uses several classes that are part of the JMapViewer download 
   (http://svn.openstreetmap.org/applications/viewer/jmapviewer).
   Another example of their use is inside JMapViewer.java.

   Usage:
      run StitchTiles 7.006835 100.5020
                // Robot building
      run StitchTiles 7.001135 100.491960
                // Oak Tree Bakery
      run StitchTiles 53.314401 -2.255305
                // UK address

*/

import java.io.*;
import java.awt.*;
import java.awt.image.*;
import javax.imageio.*;

import java.util.concurrent.atomic.*;
import org.openstreetmap.gui.jmapviewer.*;
import org.openstreetmap.gui.jmapviewer.interfaces.*;
import org.openstreetmap.gui.jmapviewer.JobDispatcher.JobThread;



public class StitchTiles implements TileLoaderListener
{
   // Robot building is the default position
  private static final double LAT_DEFAULT = 7.006835; 
  private static final double LONG_DEFAULT = 100.5020;

  private static final int ZOOM = 17;   // so roads are visible
  private static final int TILES_RADIUS = 1;   
        // tiles radius around the center tile for downloading
        // '1' means 8 tiles around the center, so 9 are downloaded altogether

  private static final String TILES_DIR = "/tiles";
        // the local directory for storing the downloaded tiles

  private static final String IMG_FNM = "tilesIm.png";
        // file for storing the combined tiles image


  private OsmTileSource.Mapnik tileSource;      // Web-based source for the maps
  private OsmFileCacheTileLoader tileLoader;    // loader which uses Web or local dir
  private MemoryTileCache tileCache;            // in-memory data structure for tiles

  // for managing the concurrent download tasks
  private JobDispatcher jobDispatcher = JobDispatcher.getInstance();   
  private AtomicInteger jobCount = new AtomicInteger(0);


  public StitchTiles(double lat, double lon) 
  {  
    tileSource = new OsmTileSource.Mapnik();

    tileLoader = new OsmFileCacheTileLoader(this, 
                             new File( System.getProperty("user.dir") + TILES_DIR));
    /* "this" means that this object is notified when a download finishes:
                tileLoadingFinished() is called;
       the second argument is the directory where downloaded tiles are
       stored
    */
    System.out.println("Tile cache dir: " + tileLoader.getCacheDirBase() + "\n");

    tileCache = new MemoryTileCache();

    // Get tile ID (x,y) for the user's specified location
    int tileSize = tileSource.getTileSize();
    int tilex = OsmMercator.LonToX(lon, ZOOM)/tileSize;
    int tiley = OsmMercator.LatToY(lat, ZOOM)/tileSize;

    loadTiles(tilex, tiley);
    reportCache(tilex, tiley);
    saveTilesImage(tilex, tiley, tileSize);
  }  // end of StitchTiles()



  private void loadTiles(int tilex, int tiley)
  // load tiles around this tile into the tiles cache
  {
    for (int x = (tilex - TILES_RADIUS); x <= (tilex + TILES_RADIUS); x++)
      for (int y = (tiley - TILES_RADIUS); y <= (tiley + TILES_RADIUS); y++)
        loadTile(x,y);

    // wait for all the downloads to finish
    while (jobCount.get() != 0) {
      try {   
        Thread.sleep(500);
      }
      catch(InterruptedException ex){}
    }
  }  // end of loadTiles()



  private void loadTile(int tx, int ty)
  /* download a tile over the Web *or* read in from a local file in TILES_DIR;
     store details in the tiles cache */
  {
    // System.out.println("--- tile at: (" + tilex + ", " + tiley + ")-------------");
    Tile tile = new Tile(tileSource, tx, ty, ZOOM);
    tileCache.addTile(tile);
    System.out.println("Added " + tile + " to cache");

    if (!tile.isLoaded()) {
      System.out.println("Starting loading " + tile);
      jobDispatcher.addJob(tileLoader.createTileLoaderJob(tileSource, tx, ty, ZOOM));  
                                           // download or read
      jobCount.incrementAndGet();
    }
    else
      System.out.println("Cached tile already loaded");
  }  // end of loadTile()



  private void reportCache(int tilex, int tiley)
  // report on tile contents inside the cache
  {
    System.out.println("\nSize of cache: " + tileCache.getTileCount());
    Tile t;
    for (int x = (tilex - TILES_RADIUS); x <= (tilex + TILES_RADIUS); x++)
      for (int y = (tiley - TILES_RADIUS); y <= (tiley + TILES_RADIUS); y++) {
        t = tileCache.getTile(tileSource, x, y, ZOOM);
        System.out.println("  " + t + " load status: " + t.isLoaded());
        System.out.println("    " + t.getUrl());
      }
  }  // end of reportCache()



  private void saveTilesImage(int tilex, int tiley, int tileSize)
  // construct and save a single image made from all the tiles
  {
    System.out.println();
    BufferedImage tilesIm = combineTiles(tilex, tiley, tileSize);
    try {  // Write image to file
      ImageIO.write(tilesIm, "png", new File(IMG_FNM));
      System.out.println("Saved tiles image to " + IMG_FNM);
    } 
    catch (IOException e) 
    {  System.out.println("Could not save tiles image to " + IMG_FNM);  }
  }  // end of saveTilesImage()



  private BufferedImage combineTiles(int tilex, int tiley, int tileSize)
  // combine all the tile images into a single image
  {
    int widthTiles = (TILES_RADIUS*2)+1;

    // create empty image
    BufferedImage im = new BufferedImage(widthTiles*tileSize, widthTiles*tileSize,
                                              BufferedImage.TYPE_INT_ARGB);
    Graphics2D g2d = im.createGraphics();

    Tile t;
    int xPos = 0;
    int yPos;
    // stitch the tiles together into an image column-by column, left-to-right
    for (int x = (tilex - TILES_RADIUS); x <= (tilex + TILES_RADIUS); x++) {
      yPos = 0;
      for (int y = (tiley - TILES_RADIUS); y <= (tiley + TILES_RADIUS); y++) {
        t = tileCache.getTile(tileSource, x, y, ZOOM);
        g2d.drawImage(t.getImage(), xPos, yPos, null);
        yPos += tileSize;
      }
      xPos += tileSize;
    }
    g2d.dispose();
    return im;
  }  // end of combineTiles()


  // ------------- TileLoaderListener methods -----------------

  public void tileLoadingFinished(Tile tile, boolean success) 
  {  System.out.println("  Finished loading " + tile + ": " + success);  
     jobCount.decrementAndGet();
  }

  public TileCache getTileCache() 
  {  return tileCache; }


  // -------------------------------------------------------

  public static void main(String[] args) 
  {  
    double lat = LAT_DEFAULT;
    double lon = LONG_DEFAULT;
    if (args.length !=2)
      System.err.println("Usage: run StitchTiles [ latitude longitude ]");
    else {
      lat = Double.parseDouble(args[0]);
      lon  = Double.parseDouble(args[1]);
    }
    System.out.printf("Using (lat, long): (%.6f, %.6f)\n", lat, lon);
    new StitchTiles(lat, lon);  
  }  // end of main()

}  // end of StitchTiles class
