
// SearchPanoramio.java
// Andrew Davison, November 2010, ad@fivedots.coe.psu.ac.th

/* Query Panoramio's API using its REST-based interface that 
   returns search results in JSON format.
   For details, see
      http://www.panoramio.com/api/data/api.html

  Use Panoramio's image search to look for images within a fixed distance 
  (NEAR_DIST) of a supplied latitude and longitude.
  e.g:
      run SearchPanoramio 7.006835 100.5020 200
                // Robot building
      run SearchPanoramio 7.001135 100.491960
                // Oak Tree Bakery
      run SearchPanoramio 53.314401 -2.255305
                // UK address

  You can search in Google maps using lat, long
      
  The first 20 matches are listed (description and URL), and the one
  located closest to the supplied latitude and longitude is 
  saved in the IMG_FNM PNG file.
*/

import java.awt.image.*;
import javax.imageio.*;
import java.net.*;
import java.io.*;

import org.json.*;


public class SearchPanoramio 
{
  private static final int NEAR_DIST = 500;   // default radius (ms)
  private static final String IMG_FNM = "nearest.png"; 


  public static void main(String args[])
  {       
    if ((args.length < 2) || (args.length > 3)) {
      System.err.println("Usage: run SearchPanoramio latitude longitude [distance radius]");
      System.exit(1);
    }

    double lat = Double.parseDouble(args[0]);
    double lon  = Double.parseDouble(args[1]);
    int radius = NEAR_DIST;
    if (args.length == 3)
      radius = Integer.parseInt(args[2]);

    BufferedImage im = imagesSearch(lat, lon, radius);
    if (im == null)
      System.out.println("No image to save");
    else {
      try {
        ImageIO.write(im, "png", new File(IMG_FNM));
        System.out.println("Saved image to " + IMG_FNM);
      }
      catch (IOException e)
      {  System.out.println("Could not save image to " + IMG_FNM);  }
    }
  }  // end of main()



  private static BufferedImage imagesSearch(double lat, double lon, int radius)
  /* query Panoramio's search for images close to (lat,lon); and
     return the closest.
      Typical search URL:
        http://www.panoramio.com/map/get_panoramas.php?
          set=public&from=0&to=20&
          minx=-180&miny=-90&
          maxx=180&maxy=90&
          size=medium&mapfilter=true
  */
  {
    System.out.printf("Searching Panoramio close to (%.6f, %.6f)\n", lat, lon);
    System.out.println("Radius (m): " + radius);
    double[] bbox = GeoMath.boundingBoxCoords(lat, lon, radius);
    System.out.printf("Bounding box: Min: (%.6f, %.6f); Max:(%.6f, %.6f)\n\n",
                                bbox[0] , bbox[1], bbox[2], bbox[3]);
    try {
      String urlStr = "http://www.panoramio.com/map/get_panoramas.php?" +
                        "set=public&from=0&to=20&" +      // full; first 20 images
                        "miny=" + bbox[0] + "&minx=" + bbox[1] + "&" +
                        "maxy=" + bbox[2] + "&maxx=" + bbox[3] + "&" +
                        "size=medium";
      String jsonStr = WebUtils.webGetString(urlStr);

      JSONObject json = new JSONObject(jsonStr);
      WebUtils.saveString("temp.json", json.toString(2) );    // indent the string

      PanoramioItem[] items = extractMatches(json);
      if (items != null) {
        int itemIdx = selectClosestItem(items, lat, lon);
        System.out.println("Closest Photo: " + items[itemIdx].getPhotoTitle());
        System.out.printf("(lat,long): (%.6f,%,6f)\n",
                     items[itemIdx].getLatitude(), items[itemIdx].getLongitude() );
        return items[itemIdx].getImage();
      }
    }
 	catch (Exception e) 
	{  System.out.println(e);  }
    return null;
  }  // end of imagesSearch()



  private static PanoramioItem[] extractMatches(JSONObject json)
  /* list the results, and return them as an array of PanoramioItem objects
  */
  {
    try {
      int count = Integer.parseInt( json.getString("count"));
      if (count > 0) {
        // list the photo titles and their URLs
        JSONArray ja = json.getJSONArray("photos");
        int numMatches = ja.length();
        System.out.println("\nNo. of matches: " + numMatches + "\n");
        if (numMatches == 0)
          return null;
        PanoramioItem[] items = new PanoramioItem[numMatches];

        for (int i = 0; i < numMatches; i++) {
          items[i] = new PanoramioItem( ja.getJSONObject(i) );

          System.out.print((i+1) + ". ");
          System.out.println("Photo Title: " + items[i].getPhotoTitle());
          System.out.printf("   (lat,long): (%.6f,%,6f)\n",
                     items[i].getLatitude(), items[i].getLongitude() );
          System.out.println("   URL: "+ items[i].getPhotoFileURL() + "\n");
        }
        return items;
      }
      else
        System.out.println("\nNo matches found");
    }
    catch (Exception e) 
    { System.out.println(e); 
      System.exit(1);
    }
    
    return null;
  } // end of extractMatches()



  private static int selectClosestItem(PanoramioItem[] items, 
                                            double lat, double lon)
  // search the Panoramio results for the image closest to (lat,lon) 
  {
    int itemIdx = 0;
    double minDist = GeoMath.distanceApart(lat, lon, 
                                     items[itemIdx].getLatitude(), 
                                     items[itemIdx].getLongitude());
    for (int i=1; i < items.length; i++) {
      double distApart = GeoMath.distanceApart(lat, lon, 
                          items[i].getLatitude(), items[i].getLongitude());
      if (distApart < minDist) {
        minDist = distApart;
        itemIdx = i;
      }
    }
    return itemIdx;
  }  // end of selectClosestItem()
  

 } // end of SearchPanoramio class
